// Bentigt GLUT!
// Fr Dev-C++ (also Mingw32) siehe http://sites.netscape.net/ptrpck/glut.htm
// mit GLUT linken: Project/Project options/Further object files or linker options:
// -lopengl32 -lglu32 -lglut32

// Korrekturen fr GLUT in Dev-C++:
#define GLUT_NO_LIB_PRAGMA
#define GLUT_NO_WARNING_DISABLE

#include <gl/glut.h>
#include "vector2d.h"

// 2D-Differentialgleichung: u' = f(u,t)
Vector2D f(double t, Vector2D u)
{
    return exp(t) * Vector2D(u.y, -u.x);
}

// exakte Lsung
void drawExact()
{
    double t;
    const double dt = 1.0e-4;
    for(t=0.0; t<5.0; t+=dt)
    {
        Vector2D u(sin(exp(t)), cos(exp(t)));
        glColor3f(0.0, 0.5, 0.5);
        u.plotX(t);
        glColor3f(0.0, 0.5, 0.0);
        u.plotY(t);
    }
}

// (einfaches) Euler-Verfahren (= Polygon)
void drawEuler()
{
    Vector2D u(sin(1), cos(1));
    const double dt = 1.0e-3;
    double t;
    for(t = 0.0; t < 5.0; t += dt)
    {
        glColor3f(0.0, 1.0, 1.0);
        u.plotX(t);
        glColor3f(0.0, 1.0, 0.0);
        u.plotY(t);
        u = u + dt*f(t, u);
    }
}

// Mittelpunktsverfahren (= verbessertes Euler-Verfahren)
void drawMidpoint()
{
    Vector2D u(sin(1), cos(1));
    const double dt = 2.0 * 1.0e-3; // fr gleichen Aufwand wie einfaches Euler
    double t = 0.0;
    while(t < 5.0)
    {
        glColor3f(0.0, 1.0, 1.0);
        u.plotX(t);
        glColor3f(0.0, 1.0, 0.0);
        u.plotY(t);

        Vector2D um = u + (0.5*dt)*f(t, u);
        t += 0.5*dt;
        u = u + dt*f(t, um);
        t += 0.5*dt;
    }
}

// Runge-Kutta-Verfahren 4. Ordnung
void drawRungeKutta()
{
    Vector2D u(sin(1), cos(1));
    const double dt = 4.0 * 1.0e-3; // fr gleichen Aufwand wie einfaches Euler
    double t = 0.0;
    while(t < 5.0)
    {
        glColor3f(0.0, 1.0, 1.0);
        u.plotX(t);
        glColor3f(0.0, 1.0, 0.0);
        u.plotY(t);

        Vector2D du1 = dt*f(t, u);
        t += 0.5*dt;
        Vector2D du2 = dt*f(t, u+0.5*du1);
        Vector2D du3 = dt*f(t, u+0.5*du2);
        t += 0.5*dt;
        Vector2D du4 = dt*f(t, u+du3);
        u = u + (1.0/6.0)*du1 + (1.0/3.0)*du2 + (1.0/3.0)*du3 + (1.0/6.0)*du4;
    }
}

void renderScene()
{
    glClear(GL_COLOR_BUFFER_BIT);

    glPointSize(2);

    glEnable(GL_POINT_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glBegin(GL_POINTS);
        drawExact();
        drawEuler();
        drawMidpoint();
        drawRungeKutta();
    glEnd();

    glFlush();
}

void changeSize(GLsizei w, GLsizei h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0.0, 5.0, -2.0, 2.0);
}

int main(int argc, char* argv[])
{
    glutInit(&argc, argv);

    glutInitWindowSize(800, 600);
    glutInitWindowPosition(20, 25);

    // Standard: glutInitDisplayMode(GLUT_RGB | GLUT_SINGLE);
    glutCreateWindow("DGL-Demo");

    glutDisplayFunc(renderScene);
    glutReshapeFunc(changeSize);

    glutMainLoop();

    return 0; // nie erreicht
}
